from .base import V4Client


class GeneralAPI(V4Client):
    async def version(self):
        path = self.make_resource("/version")
        result = await self.get(path)
        return result.get("api_version")



import smtplib

from typing import Optional, Union, Type, List, Dict, Any, Tuple

from domjudge_tool_cli.utils.email.helper import EmailContext


class SMTP:
    host: str = "localhost"
    port: int = 25
    use_ssl: bool = False
    timeout: Optional[int] = None
    username: Optional[str] = None
    password: Optional[str] = None
    connection: Optional[Union[smtplib.SMTP_SSL, smtplib.SMTP]]

    def __init__(
        self,
        host: str = "localhost",
        port: int = 25,
        use_ssl: bool = False,
        timeout: Optional[int] = None,
        username: Optional[str] = None,
        password: Optional[str] = None,
    ):
        self.host = host
        self.port = port
        self.use_ssl = use_ssl
        self.timeout = timeout
        self.username = username
        self.password = password
        self.connection = None

    @property
    def connection_class(self) -> Union[Type[smtplib.SMTP_SSL], Type[smtplib.SMTP]]:
        return smtplib.SMTP_SSL if self.use_ssl else smtplib.SMTP

    def open(self):
        connection_params = dict()
        if self.timeout is not None:
            connection_params["timeout"] = self.timeout

        self.connection = self.connection_class(
            self.host, self.port, **connection_params
        )
        self.connection.ehlo()
        self.connection.starttls()

        if self.username and self.password:
            self.connection.login(self.username, self.password)

    def close(self):
        if self.connection is None:
            return
        try:
            try:
                self.connection.quit()
            except Exception:
                self.connection.close()
                raise
        finally:
            self.connection = None

    def send_message(
        self,
        from_email: str,
        to_address: List[str],
        message: EmailContext,
        **kwargs: Dict[str, Any],
    ) -> Dict[str, Tuple[int, bytes]]:
        msg = message.mime(from_email, to_address, **kwargs)
        return self.connection.sendmail(
            from_email,
            to_address,
            msg.as_string(),
        )



from io import BytesIO
from typing import List, Optional

from domjudge_tool_cli.models import Team

from .base import V4Client


class TeamsAPI(V4Client):
    async def all_teams(
        self,
        cid: str,
    ) -> List[Team]:
        path = self.make_resource(f"/contests/{cid}/teams")
        result = await self.get(path)
        return list(map(lambda it: Team(**it), result))

    async def team(self, cid: str, id: str) -> Team:
        path = self.make_resource(f"/contests/{cid}/teams/{id}")
        result = await self.get(path)
        return Team(**result)



from typing import List, Optional

from pydantic import BaseModel


class Problem(BaseModel):
    ordinal: int
    id: str
    short_name: str
    label: str
    time_limit: int
    externalid: str
    name: str
    rgb: Optional[str] = None
    color: Optional[str] = None
    test_data_count: int


class ProblemItem(BaseModel):
    id: str
    time_limit: int
    test_data_count: int
    name: str
    export_file_path: Optional[str]



from io import BytesIO
from typing import List, Optional

from domjudge_tool_cli.models import Problem

from .base import V4Client


class ProblemsAPI(V4Client):
    async def all_problems(
        self,
        cid: str,
    ) -> List[Problem]:
        path = self.make_resource(f"/contests/{cid}/problems")
        result = await self.get(path)
        return list(map(lambda it: Problem(**it), result))

    async def problem(self, cid: str, id: str) -> Problem:
        path = self.make_resource(f"/contests/{cid}/problems/{id}")
        result = await self.get(path)
        return Problem(**result)



from pathlib import Path
from typing import Optional

import typer

from .commands import general, problems, scoreboard, submissions, users, emails

__version__ = "0.1.0"

app = typer.Typer()

app.add_typer(general.app, name="general")
app.add_typer(users.app, name="users")
app.add_typer(scoreboard.app, name="scoreboard")
app.add_typer(submissions.app, name="submissions")
app.add_typer(problems.app, name="problems")
app.add_typer(emails.app, name="emails")


@app.callback()
def main(
    verbose: bool = typer.Option(
        False,
        "--verbose",
        "-v",
    ),
    config: Optional[Path] = typer.Option(
        None,
        help="Dom server config JSON file",
        envvar="DOMSERVER_CONFIG",
    ),
):
    if config:
        if verbose:
            typer.echo(f"Dom server config file: {config}")
        general.general_state["config"] = config



import asyncio
from typing import List, Optional

import typer

from domjudge_tool_cli.commands.general import general_state, get_or_ask_config

from ._submissions import (
    download_contest_files,
    download_submission_files,
    get_submissions,
)

app = typer.Typer()


@app.command()
def submission_list(
    cid: str,
    language_id: Optional[str] = None,
    strict: Optional[bool] = False,
    ids: Optional[List[str]] = None,
):
    submission_ids = None
    if ids:
        submission_ids = ids.split(",")

    client = get_or_ask_config(general_state["config"])
    asyncio.run(get_submissions(client, cid, language_id, strict, submission_ids))


@app.command()
def submission_file(
    cid: str,
    id: str,
    mode: int = 2,
    path: Optional[str] = None,
    strict: Optional[bool] = False,
):
    client = get_or_ask_config(general_state["config"])
    asyncio.run(download_submission_files(client, cid, id, mode, path, strict))


@app.command()
def contest_files(
    cid: str,
    mode: int = 2,
    path: Optional[str] = None,
):
    client = get_or_ask_config(general_state["config"])
    asyncio.run(download_contest_files(client, cid, mode, path))



from typing import Optional

from pydantic import BaseModel


class Affiliation(BaseModel):
    id: Optional[str]
    shortname: str
    name: str
    country: str
    team_affiliation: Optional[str]



from typing import List, Optional

import aiofiles
from aiofiles import os as aio_os

from domjudge_tool_cli.models import Submission, SubmissionFile

from .base import V4Client


class SubmissionsAPI(V4Client):
    async def all_submissions(
        self,
        cid: str,
        language_id: Optional[str] = None,
        strict: Optional[bool] = False,
        ids: Optional[List[str]] = None,
    ) -> List[Submission]:
        path = self.make_resource(f"/contests/{cid}/submissions")
        params = dict()

        if ids:
            params["ids[]"] = ids

        if strict:
            params["strict"] = strict

        if language_id:
            params["language_id"] = language_id

        result = await self.get(
            path,
            params if params else None,
        )

        return list(map(lambda it: Submission(**it), result))

    async def submission(self, cid: str, id: str) -> Submission:
        path = self.make_resource(f"/contests/{cid}/submissions/{id}")
        result = await self.get(path)
        return Submission(**result)

    async def submission_files(
        self,
        cid: str,
        id: str,
        filename: str,
        file_path: Optional[str] = None,
        strict: Optional[bool] = False,
    ) -> any:
        is_dir = await aio_os.path.isdir(file_path)
        if not is_dir:
            await aio_os.makedirs(file_path, exist_ok=True)

        path = self.make_resource(f"/contests/{cid}/submissions/{id}/files")
        result = await self.get_file(path)
        async with aiofiles.open(f"{file_path}/{id}-{filename}.zip", "wb") as f:
            await f.write(result)

    async def submission_file_name(
        self,
        cid: str,
        id: str,
    ) -> SubmissionFile:

        path = self.make_resource(f"/contests/{cid}/submissions/{id}/source-code")
        result = await self.get(path)
        return SubmissionFile(**result[0])



import asyncio
from typing import Any, List, Optional

import typer
from tablib import Dataset

from domjudge_tool_cli.models import DomServerClient, Submission
from domjudge_tool_cli.services.v4 import ProblemsAPI, SubmissionsAPI, TeamsAPI


def gen_submission_dataset(submissions: List[Any]) -> Dataset:
    dataset = Dataset()
    for idx, submission in enumerate(submissions):
        if idx == 0:
            dataset.headers = submission.dict().keys()

        dataset.append(submission.dict().values())

    return dataset


def print_submissions_table(submissions: List[Submission]):
    dataset = gen_submission_dataset(submissions)
    # ["id", "team_id", "problem_id", "language_id",
    # "files", "entry_point", "time", "contest_time", "externalid"]
    for rm_key in ["externalid", "contest_time"]:
        del dataset[rm_key]
    typer.echo(dataset.export("cli", tablefmt="simple"))


def file_path(cid, mode, path, team, problem):
    if mode == 1:
        file_path = f"Teams/{team.name}/Problems/{problem.short_name}/Submissions"
    elif mode == 2:
        file_path = f"Problems/{problem.short_name}/Teams/{team.name}/Submissions"
    else:
        file_path = f"contests/{cid}/submissions"

    if path:
        file_path = f"{path}/{file_path}"

    return file_path


def index_by_id(objs):
    data = dict()
    for obj in objs:
        data[obj.id] = obj
    return data


async def get_submissions(
    client: DomServerClient,
    cid: str,
    language_id: Optional[str] = None,
    strict: Optional[bool] = False,
    ids: Optional[List[str]] = None,
):
    async with SubmissionsAPI(**client.api_params) as api:
        submissions = await api.all_submissions(cid)

        print_submissions_table(submissions)


async def download_submission_files(
    client: DomServerClient,
    cid: str,
    id: str,
    mode: int,
    path_prefix: Optional[str] = None,
    strict: Optional[bool] = False,
):
    async with SubmissionsAPI(**client.api_params) as api:
        submission = await api.submission(cid, id)

        async with TeamsAPI(**client.api_params) as team_api:
            team = await team_api.team(cid, submission.team_id)

        async with ProblemsAPI(**client.api_params) as problem_api:
            problem = await problem_api.problem(cid, submission.problem_id)

        submission_file = await api.submission_file_name(cid, id)
        submission_filename = submission_file.filename.split(".")[0]

        path = file_path(cid, mode, path_prefix, team, problem)
        await api.submission_files(cid, id, submission_filename, path)


async def download_contest_files(
    client: DomServerClient,
    cid: str,
    mode: int,
    path_prefix: Optional[str] = None,
):
    async with SubmissionsAPI(**client.api_params) as api:
        submissions = await api.all_submissions(cid)

        async with TeamsAPI(**client.api_params) as team_api:
            teams = await team_api.all_teams(cid)
            teams_mapping = index_by_id(teams)

        async with ProblemsAPI(**client.api_params) as problem_api:
            problems = await problem_api.all_problems(cid)
            problems_mapping = index_by_id(problems)

        async def get_source_codes(submission) -> None:
            id = submission.id
            team = teams_mapping[submission.team_id]
            problem = problems_mapping[submission.problem_id]

            submission_file = await api.submission_file_name(cid, id)
            submission_filename = submission_file.filename.split(".")[0]

            path = file_path(cid, mode, path_prefix, team, problem)
            await api.submission_files(cid, id, submission_filename, path)

        with typer.progressbar(submissions) as progress:
            for task in progress:
                await get_source_codes(task)



import random
import string


def gen_password(length=None, pattern=None) -> str:
    if not length:
        length = 10

    if not pattern:
        pattern = string.ascii_letters + string.digits

    return "".join(random.choices(pattern, k=length))



import os

from string import Template
from io import TextIOWrapper
from pathlib import Path
from typing import Optional, Union, Tuple, List, Dict, Any
from enum import Enum
from email.mime.text import MIMEText
from email.utils import formataddr, formatdate, getaddresses, make_msgid


class FileType(str, Enum):
    TEXT = ".txt"
    HTML = ".html"


def load_template(
    path: Union[str, TextIOWrapper, Path],
) -> Optional[Tuple[Template, str]]:
    file_name = os.path.basename(path)
    _, extension = os.path.splitext(file_name)

    with open(path, "r", encoding="utf-8") as f:
        txt = f.read()
        template = Template(txt)
        return template, extension


class EmailContext:
    subject_template: Template
    body_template: Template
    body_file_type: FileType

    def __init__(
        self,
        template_dir: str,
    ):
        path = Path(template_dir)

        assert path.exists() and path.is_dir(), f"No such directory {template_dir}."

        subject_templates = list(path.glob("subject.txt"))
        assert subject_templates, f"No such file subject.txt in {template_dir}."
        self.subject_template, _ = load_template(subject_templates[0])

        body_templates = list(path.glob("body.html")) + list(path.glob("body.txt"))
        assert (
            body_templates
        ), f"No such file subject.html or subject.txt in {template_dir}."
        self.body_template, self.body_file_type = load_template(body_templates[0])

    def render_subject(self, **kwargs) -> str:
        return self.subject_template.substitute(**kwargs)

    def render_body(self, **kwargs) -> str:
        return self.body_template.substitute(**kwargs)

    def mime(
        self,
        from_email: str,
        to_address: List[str],
        **kwargs: Dict[str, Any],
    ) -> MIMEText:
        _, domain = from_email.split("@")
        mail_content_type = "html" if self.body_file_type == FileType.HTML else "plain"
        subject = self.render_subject(**kwargs)
        content = self.render_body(**kwargs)
        mime = MIMEText(content, mail_content_type, "utf-8")
        mime["Subject"] = subject
        mime["From"] = from_email
        mime["Date"] = formatdate(localtime=True)
        mime["Message-ID"] = make_msgid(domain=domain)
        mime["To"] = ",".join(to_address)

        return mime



from typing import Optional, Union

from tablib import Dataset

import typer

from domjudge_tool_cli.utils.email import helper, smtp
from domjudge_tool_cli.models import CreateUser

app = typer.Typer()


@app.command()
def send_user_accounts(
    file: typer.FileText = typer.Argument(...),
    template_dir: str = typer.Argument(...),
    host: Optional[str] = typer.Option("localhost"),
    port: Optional[int] = typer.Option(25),
    from_email: Optional[str] = typer.Option("noreply@localhost"),
    use_ssl: Optional[bool] = typer.Option(False),
    format: Optional[str] = typer.Option("csv"),
    timeout: Optional[int] = None,
    username: Optional[str] = None,
    password: Optional[str] = None,
):
    input_file = file
    if format == "csv":
        input_file = file.read().replace("\ufeff", "")

    dataset = Dataset().load(input_file, format=format)
    context = helper.EmailContext(template_dir)
    _, domain = from_email.split("@")
    connection = smtp.SMTP(
        host,
        port,
        use_ssl,
        timeout,
        username,
        password,
    )
    connection.open()
    with typer.progressbar(dataset.dict) as progress:
        for item in progress:
            item["email"] = None if not item.get("email") else item["email"]
            user = CreateUser(**item)
            to_email = f"{user.username}@{domain}" if not user.email else user.email
            connection.send_message(from_email, [to_email], context, **item)

    connection.close()



from typing import Any, Dict, List, Optional

import httpx
from pydantic import BaseModel, HttpUrl


class DomServerClient(BaseModel):
    host: HttpUrl
    username: str
    password: str
    disable_ssl: bool = False
    timeout: Optional[float] = None
    max_connections: Optional[int] = None
    max_keepalive_connections: Optional[int] = None
    category_id: Optional[int] = None
    affiliation_id: Optional[int] = None
    affiliation_country: Optional[str] = "TWN"
    user_roles: Optional[List[int]] = None

    @property
    def get_timeout(self) -> Optional["httpx.Timeout"]:
        if self.timeout:
            return httpx.Timeout(self.timeout)

    @property
    def get_limits(self) -> Optional["httpx.Limits"]:
        if self.max_connections or self.max_keepalive_connections:
            return httpx.Limits(
                max_connections=self.max_connections,
                max_keepalive_connections=self.max_keepalive_connections,
            )

    @property
    def api_params(self) -> Dict[str, Any]:
        return {
            "host": self.host,
            "username": self.username,
            "password": self.password,
            "disable_ssl": self.disable_ssl,
            "timeout": self.get_timeout,
            "limits": self.get_limits,
        }



import asyncio
import os
from typing import List, Optional

import typer

from domjudge_tool_cli.commands.general import general_state, get_or_ask_config

from ._users import (
    UserExportFormat,
    create_teams_and_users,
    delete_teams_and_users,
    get_user,
    get_users,
)

app = typer.Typer()


@app.command()
def user_list(
    ids: Optional[str] = typer.Option(
        None,
        help="user_id1,user_id2,user_id3",
    ),
    team_id: Optional[str] = None,
    format: Optional[UserExportFormat] = None,
    file: Optional[typer.FileBinaryWrite] = typer.Option(
        None,
        help="Export file name",
    ),
):
    user_ids = None
    if ids:
        user_ids = ids.split(",")

    client = get_or_ask_config(general_state["config"])
    asyncio.run(get_users(client, user_ids, team_id, format, file))


@app.command()
def user(id: str):
    client = get_or_ask_config(general_state["config"])
    asyncio.run(get_user(client, id))


@app.command()
def import_users_teams_example():
    import domjudge_tool_cli

    file_name = "import-users-teams.csv"
    file_path = os.path.join(
        domjudge_tool_cli.__path__[0],
        "templates",
        "csv",
        file_name,
    )
    new_file_path = os.path.join(os.getcwd(), file_name)
    with open(file_path, encoding="utf-8") as template_file:
        content = template_file.read()

    with open(new_file_path, "w", encoding="utf-8") as file:
        file.write(content)

    typer.echo(new_file_path)


@app.command()
def import_users_teams(
    file: typer.FileText = typer.Argument(...),
    category_id: Optional[int] = typer.Option(None),
    affiliation_id: Optional[int] = typer.Option(None),
    user_roles: Optional[List[int]] = typer.Option(None),
    enabled: bool = typer.Option(True),
    format: Optional[UserExportFormat] = None,
    ignore_existing: bool = typer.Option(False),
    delete_existing: bool = typer.Option(False),
    password_length: Optional[int] = typer.Option(None),
    password_pattern: Optional[str] = typer.Option(None),
):
    client = get_or_ask_config(general_state["config"])
    asyncio.run(
        create_teams_and_users(
            client,
            file,
            category_id,
            affiliation_id,
            user_roles,
            enabled,
            format,
            ignore_existing,
            delete_existing,
            password_length,
            password_pattern,
        ),
    )


@app.command()
def rm_teams_and_users(
    include: Optional[List[str]] = typer.Option(None),
    exclude: Optional[List[str]] = typer.Option(None),
):
    client = get_or_ask_config(general_state["config"])
    asyncio.run(
        delete_teams_and_users(
            client,
            include,
            exclude,
        ),
    )



from typing import List, Optional

from pydantic import BaseModel, EmailStr


class User(BaseModel):
    last_login_time: Optional[str]
    first_login_time: Optional[str]
    team: Optional[str]
    roles: List[str]
    id: str
    username: str
    name: str
    email: Optional[EmailStr]
    last_ip: Optional[str]
    ip: Optional[str]
    enabled: bool
    team_id: Optional[str]


class CreateUser(BaseModel):
    username: str
    name: str
    email: Optional[EmailStr] = None
    password: Optional[str] = None
    affiliation: Optional[str] = None



from typing import List, Optional

from domjudge_tool_cli.models import User

from .base import V4Client


class UsersAPI(V4Client):
    async def all_users(
        self,
        ids: Optional[List[str]] = None,
        team_id: Optional[str] = None,
    ) -> List[User]:
        path = self.make_resource("/users")
        params = dict()

        if ids:
            params["ids[]"] = ids

        if team_id:
            params["team_id"] = team_id

        result = await self.get(
            path,
            params if params else None,
        )
        return list(map(lambda it: User(**it), result))

    async def get_user(
        self,
        id: str,
    ) -> User:
        path = self.make_resource(f"/users/{id}")

        result = await self.get(path)
        return User(**result)



from typing import Any, Dict, Optional

import httpx


class BaseClient:
    def __init__(
        self,
        host: str,
        disable_ssl: Optional[bool] = None,
        timeout: Optional[httpx.Timeout] = None,
        limits: Optional[httpx.Limits] = None,
    ):
        self.host = host
        self._parameters = dict(base_url=host)

        if disable_ssl:
            self._parameters["verify_ssl"] = False

        if timeout:
            self._parameters["timeout"] = timeout

        if limits:
            self._parameters["limits"] = limits

        self.client = self.new_client()

    def new_client(self) -> "httpx.AsyncClient":
        return httpx.AsyncClient(**self._parameters)

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.client.__aexit__(exc_type, exc_val, exc_tb)


class APIClient(BaseClient):
    def __init__(
        self,
        host: str,
        username: str,
        password: str,
        disable_ssl: Optional[bool] = None,
        timeout: Optional[httpx.Timeout] = None,
        limits: Optional[httpx.Limits] = None,
    ):
        super().__init__(host, disable_ssl, timeout, limits)
        self.username = username
        self.password = password
        self._parameters = dict(
            base_url=host,
            auth=httpx.BasicAuth(username, password),
        )
        self.client = self.new_client()

    async def get(
        self,
        path: str,
        params: Optional[Dict[str, Any]] = None,
    ) -> Any:
        r = await self.client.get(path, params=params)  # type: httpx.Response
        r.raise_for_status()
        return r.json()

    async def get_file(
        self,
        path: str,
        params: Optional[Dict[str, Any]] = None,
    ) -> Any:
        r = await self.client.get(path, params=params)  # type: httpx.Response
        r.raise_for_status()
        return r.content


class WebClient(BaseClient):
    def __init__(
        self,
        host: str,
        username: str,
        password: str,
        disable_ssl: Optional[bool] = None,
        timeout: Optional[httpx.Timeout] = None,
        limits: Optional[httpx.Limits] = None,
    ):
        self.username = username
        self.password = password
        super().__init__(host, disable_ssl, timeout, limits)

    async def get(
        self,
        path: str,
        params: Optional[Dict[str, Any]] = None,
    ) -> httpx.Response:
        r = await self.client.get(  # type: httpx.Response
            path,
            params=params,
            allow_redirects=True,
        )
        r.raise_for_status()
        return r

    async def post(
        self,
        path: str,
        body: Optional[Dict[str, Any]] = None,
    ) -> httpx.Response:
        r = await self.client.post(  # type: httpx.Response
            path,
            data=body,
            allow_redirects=True,
        )
        r.raise_for_status()
        return r



from .affiliation import Affiliation
from .domserver import DomServerClient
from .problem import Problem, ProblemItem
from .submission import Submission, SubmissionFile
from .team import Team
from .user import CreateUser, User

__all__ = (
    "DomServerClient",
    "User",
    "CreateUser",
    "Submission",
    "SubmissionFile",
    "Team",
    "Problem",
    "ProblemItem",
    "Affiliation",
)



import asyncio
from enum import Enum
from typing import List, Optional, Tuple

from bs4 import BeautifulSoup

from domjudge_tool_cli.models import Affiliation, CreateUser, ProblemItem
from domjudge_tool_cli.services.api_client import WebClient


class HomePath(str, Enum):
    JURY = "/jury"
    LOGIN = "/login"


class UserPath(str, Enum):
    LIST = "/jury/users"
    ADD = "/jury/users/add"
    EDIT = "/jury/users/%s/edit"


class TeamPath(str, Enum):
    LIST = "/jury/teams"
    ADD = "/jury/teams/add"


class AffiliationPath(str, Enum):
    LIST = "/jury/affiliations"
    ADD = "/jury/affiliations/add"


class ProblemPath(str, Enum):
    LIST = "/jury/problems"
    ADD = "/jury/problems/add"


def _get_input_fields(page: str) -> dict:
    soup = BeautifulSoup(page, "html.parser")

    data = {ele.get("name"): ele.get("value") for ele in soup.select("input")}

    select_tags = soup.select("select")
    for tag in select_tags:
        option = tag.select_one("option[selected]")
        data[tag.get("name")] = option.get("value") if option else None

    data.pop(None, None)  # remove no name fields
    return data


class DomServerWeb(WebClient):
    async def login(self) -> None:
        login_form = await self.get(HomePath.LOGIN.value)
        data = {
            **_get_input_fields(login_form.text),
            "_username": self.username,
            "_password": self.password,
        }
        res = await self.post(HomePath.LOGIN.value, body=data)

        assert res.url.path == HomePath.JURY.value, "Login fail."

    async def create_team_and_user(
        self,
        user: CreateUser,
        category_id: int,
        affiliation_id: int,
        enabled: bool = True,
    ) -> Tuple[str, str]:
        res = await self.get(TeamPath.ADD.value)

        data = {
            **_get_input_fields(res.text),
            "team[name]": user.username,
            "team[displayName]": user.name,
            "team[affiliation]": str(affiliation_id),
            "team[enabled]": "1" if enabled else "0",
            "team[addUserForTeam]": "1",  # '1' -> Yes
            "team[users][0][username]": user.username,
            "team[category]": str(category_id),
        }

        if "team[contests][]" in data and data["team[contests][]"] is None:
            data.pop("team[contests][]")

        res = await self.post(TeamPath.ADD.value, body=data)
        assert res.url.path != TeamPath.ADD.value, "Team create fail."
        team_id = res.url.path.split("/")[-1]

        res = await self.get(res.url.path)  # Go to team view page.

        soup = BeautifulSoup(res.text, "html.parser")
        user_link = soup.select_one(".container-fluid a")
        user_id = user_link["href"].split("/")[-1]

        return team_id, user_id

    async def set_user_password(
        self,
        user_id: str,
        password: str,
        user_roles: List[int],
        enabled: bool = True,
    ) -> None:
        url = UserPath.EDIT.value % user_id

        res = await self.get(url)

        user_roles_data = list(map(str, user_roles))

        data = {
            **_get_input_fields(res.text),
            "user[plainPassword]": password,
            "user[enabled]": "1" if enabled else "0",
            "user[user_roles][]": user_roles_data,
        }

        res = await self.post(url, body=data)
        res.raise_for_status()

        assert res.url.path != url, "User set password fail."

    async def delete_users(self, include=None, exclude=None):
        include = include if include else []
        include = set(map(lambda it: it.lower(), include))
        exclude = exclude if exclude else []
        exclude = set(map(lambda it: it.lower(), exclude))
        res = await self.get(UserPath.LIST.value)
        res.raise_for_status()

        soup = BeautifulSoup(res.text, "html.parser")
        links = []
        for row in soup.select("table tbody tr"):
            name = row.select("a")[0].text.strip()
            lower_name = name.lower()
            if lower_name not in include or lower_name in exclude:
                continue

            link = row.select("a")[-1]["href"]
            links.append(self.post(link))

        for task in asyncio.as_completed(links):
            res = await task
            res.raise_for_status()

    async def delete_teams(self, include=None, exclude=None):
        include = include if include else []
        include = set(map(lambda it: it.lower(), include))
        exclude = exclude if exclude else []
        exclude = set(map(lambda it: it.lower(), exclude))
        res = await self.get(TeamPath.LIST.value)
        res.raise_for_status()

        soup = BeautifulSoup(res.text, "html.parser")
        links = []
        for row in soup.select("table tbody tr"):
            name = row.select("a")[2].text.strip()
            lower_name = name.lower()
            if lower_name not in include or lower_name in exclude:
                continue
            link = row.select("a")[-2]["href"]
            links.append(self.post(link))

        for task in asyncio.as_completed(links):
            res = await task
            res.raise_for_status()

    async def create_affiliation(
        self,
        shortname: str,
        name: str,
        country: str = "TWN",
    ) -> Affiliation:
        res = await self.get(AffiliationPath.ADD.value)

        data = {
            **_get_input_fields(res.text),
            "team_affiliation[shortname]": shortname,
            "team_affiliation[name]": name,
            "team_affiliation[country]": country,
            "team_affiliation[comments]": "",
        }

        res = await self.post(AffiliationPath.ADD.value, body=data)
        assert res.url.path != AffiliationPath.ADD.value, "Affiliation create fail."
        affiliation_id = res.url.path.split("/")[-1]

        return Affiliation(
            id=affiliation_id,
            shortname=shortname,
            name=name,
            country=country,
        )

    async def get_affiliations(self) -> List[Affiliation]:
        res = await self.get(AffiliationPath.LIST.value)
        res.raise_for_status()

        soup = BeautifulSoup(res.text, "html.parser")
        objs = []
        for row in soup.select("table tbody tr"):
            affiliation_id = row.select("td a")[0].text.strip()
            shortname = row.select("td a")[1].text.strip()
            name = row.select("td a")[2].text.strip()
            country = row.select("td a")[3].img.get("alt", "").strip()
            obj = Affiliation(
                id=affiliation_id,
                shortname=shortname,
                name=name,
                country=country,
            )
            objs.append(obj)

        return objs

    async def get_affiliation(self, name) -> Optional[Affiliation]:
        affiliations = await self.get_affiliations()

        for it in affiliations:
            if it.name == name or it.shortname == name:
                return it

        return None

    async def get_problems(
        self,
        exclude: Optional[List[str]] = None,
    ) -> List[ProblemItem]:
        res = await self.get(ProblemPath.LIST.value)
        res.raise_for_status()

        soup = BeautifulSoup(res.text, "html.parser")
        objs = []
        for row in soup.select("table tbody tr"):
            problem_id = row.select("td a")[0].text.strip()
            name = row.select("td a")[1].text.strip()
            time_limit = row.select("td a")[3].text.strip()
            test_data_count = row.select("td a")[6].text.strip()
            export_file_path = str(row.select("td a")[7]["href"]).strip()

            if problem_id in exclude:
                continue

            obj = ProblemItem(
                id=problem_id,
                name=name,
                time_limit=time_limit,
                test_data_count=test_data_count,
                export_file_path=export_file_path,
            )
            objs.append(obj)

        return objs



from typing import Dict, List, Optional

from pydantic import BaseModel


class Submission(BaseModel):
    language_id: Optional[str]
    time: Optional[str]
    contest_time: Optional[str]
    id: str
    externalid: Optional[str]
    team_id: str
    problem_id: str
    entry_point: Optional[str]
    files: Optional[List[Dict[str, str]]]
    submission_id: Optional[str]
    filename: Optional[str]
    source: Optional[str]


class SubmissionFile(BaseModel):
    id: str
    submission_id: Optional[str]
    filename: Optional[str]
    source: Optional[str]



import asyncio
import logging
from pathlib import Path
from typing import List, Optional

import typer

from domjudge_tool_cli.models import DomServerClient

from ._check import (
    check_login_website,
    create_config,
    get_version,
    read_config,
    update_config,
)

app = typer.Typer()
general_state = {
    "config": None,
}


def ask_want_to_config():
    host = typer.prompt("What's your dom server host URL?", type=str)
    username = typer.prompt(
        "What's your dom server username?"
        " must be `admin`, `api_reader`, `api_writer` roles.?",
        type=str,
    )
    password = typer.prompt(
        "What's your dom server password?",
        type=str,
        hide_input=True,
    )
    disable_ssl = typer.confirm("Are you want to disable verify the SSL?")
    timeout = typer.prompt(
        "Setup API timeout?",
        type=float,
        default=None,
        show_default=True,
    )
    save = typer.confirm("Are you want to save a config file?")
    if save:
        return create_config(
            host=host,
            username=username,
            password=password,
            disable_ssl=disable_ssl,
            timeout=timeout,
        )

    return DomServerClient(
        host=host,
        username=username,
        password=password,
        disable_ssl=disable_ssl,
        timeout=timeout,
    )


def get_or_ask_config(path: Optional[Path] = None) -> DomServerClient:
    try:
        return read_config(path)
    except Exception as e:
        logging.warning(e)
        return ask_want_to_config()


@app.command()
def check(
    host: Optional[str] = typer.Option(
        None, help="Dom server host URL.", show_default=False
    ),
    username: Optional[str] = typer.Option(
        None,
        help="Dom server user, must be `admin`, `api_reader`, `api_writer` roles.",
        show_default=False,
    ),
    password: Optional[str] = typer.Option(
        None,
        help="Dom server user password.",
        show_default=False,
    ),
):
    if host and username and password:
        client = DomServerClient(
            host=host,
            username=username,
            password=password,
        )
    else:
        client = get_or_ask_config(general_state["config"])

    asyncio.run(get_version(client))
    asyncio.run(check_login_website(client))


@app.command()
def contest_config(
    category_id: Optional[int] = typer.Argument(None, show_default=False),
    affiliation_id: Optional[int] = typer.Argument(None, show_default=False),
    user_roles: Optional[List[int]] = typer.Option(
        None,
        help="ex: role_id,role_id2,role_id3",
        show_default=False,
    ),
):
    client = get_or_ask_config(general_state["config"])

    update_config(
        client,
        category_id=category_id,
        affiliation_id=affiliation_id,
        user_roles=user_roles,
    )


@app.command()
def config(
    host: str = typer.Argument(..., help="Dom server host URL."),
    username: str = typer.Option(
        ...,
        help="Dom server user, must be `admin`, `api_reader`, `api_writer` roles.",
        prompt=True,
    ),
    password: str = typer.Option(
        ..., help="Dom server user password.", prompt=True, hide_input=True
    ),
    disable_ssl: Optional[bool] = typer.Option(None),
    timeout: Optional[float] = typer.Option(None),
    max_connections: Optional[int] = typer.Option(None),
    max_keepalive_connections: Optional[int] = typer.Option(None),
):
    create_config(
        host=host,
        username=username,
        password=password,
        disable_ssl=disable_ssl,
        timeout=timeout,
        max_connections=max_connections,
        max_keepalive_connections=max_keepalive_connections,
    )



import csv
from typing import Optional

import httpx
import typer
from bs4 import BeautifulSoup

from domjudge_tool_cli.commands.general import general_state, get_or_ask_config

app = typer.Typer()


def titles(items):
    headers = ["Rank", "TeamAffiliation", "TeamName", "SolvedCount", "Score"]
    for item in items:
        title = item["title"]
        if title.startswith("problem "):
            headers.append(title[8:])
    return headers


def scores(element):
    data = []
    # Rank
    data.append(element.find("td", class_="scorepl").text.strip())
    # TeamAffiliation
    data.append(
        element.find("td", class_="scoretn").find("span", class_="univ").text.strip()
    )
    # TeamName
    data.append(element.find("td", class_="scoretn").find("span").text.split()[-1])
    # SolvedCount
    data.append(element.find("td", class_="scorenc").text.strip())
    # Score
    data.append(element.find("td", class_="scorett").text.strip())
    # Problem Score
    for el in element.find_all("td", class_="score_cell"):
        s = el.text.strip().split()
        if len(s) == 0:
            data.append("")
        elif len(s) == 2:
            data.append("{} {}".format(*s))
        elif len(s) == 3:
            data.append("{}/{} {}".format(*s))
    return data


def summary(element):
    data = [""] * 3
    data.append(element.find("td", class_="scorenc").text.strip())
    data.append("")
    for el in element.find_all("td")[3:]:
        data.append("/".join(el.text.split()))
    return data


@app.command()
def export(
    cid: int,
    filename: str = "export",
    url: Optional[str] = None,
    path_prefix: Optional[str] = None,
):
    if not url:
        client = get_or_ask_config(general_state["config"])
        url = f"{client.host}/public?static=1"

    cookies = None
    if cid:
        cookies = {"domjudge_cid": f"{cid}"}

    res = httpx.get(url, cookies=cookies).content
    soup = BeautifulSoup(res, "html.parser")

    data = []
    data.append(titles(soup.find("tr", class_="scoreheader").find_all("th")))

    elements = soup.find("table", class_="scoreboard").find("tbody").find_all("tr")
    for element in elements:
        if element.find("td", id="scoresummary"):
            data.append(summary(element))
        else:
            data.append(scores(element))

    file_path = f"{filename}.csv"
    if path_prefix:
        file_path = f"{path_prefix}/{file_path}"

    with open(file_path, "w", newline="") as csvfile:
        writer = csv.writer(csvfile)
        for row in data:
            writer.writerow(row)



import typer

from enum import Enum
from typing import Any, List, Optional

from tablib import Dataset

from domjudge_tool_cli.models import CreateUser, DomServerClient, User
from domjudge_tool_cli.services.v4 import DomServerWeb, UsersAPI
from domjudge_tool_cli.utils.password import gen_password


def gen_user_dataset(users: List[Any]) -> Dataset:
    dataset = Dataset()
    for idx, user in enumerate(users):
        if idx == 0:
            dataset.headers = user.dict().keys()

        dataset.append(user.dict().values())

    return dataset


class UserExportFormat(str, Enum):
    JSON = "json"
    CSV = "csv"

    def export(
        self,
        users: List[Any],
        file: Optional[typer.FileBinaryWrite] = None,
        name: Optional[str] = None,
    ) -> str:
        dataset = gen_user_dataset(users)
        if file:
            file.write(dataset.export(self.value))
            return file.name
        else:
            if not name:
                name = f"export_users.{self.value}"
            else:
                name = f"{name}.{self.value}"

            with open(name, "w") as f:
                f.write(dataset.export(self.value))
                return name


def print_users_table(users: List[User]):
    dataset = gen_user_dataset(users)
    for rm_key in ["last_login_time", "first_login_time", "roles", "last_ip", "ip"]:
        del dataset[rm_key]
    typer.echo(dataset.export("cli", tablefmt="simple"))


async def get_users(
    client: DomServerClient,
    ids: Optional[List[str]] = None,
    team_id: Optional[str] = None,
    format: Optional[UserExportFormat] = None,
    file: Optional[typer.FileBinaryWrite] = None,
):
    async with UsersAPI(**client.api_params) as api:
        users = await api.all_users(ids, team_id)

    if ids:
        users = list(filter(lambda obj: obj.id in ids, users))

    if team_id:
        users = list(filter(lambda obj: obj.team_id == team_id, users))

    if format:
        format.export(users, file)
    else:
        print_users_table(users)


async def get_user(
    client: DomServerClient,
    id: str,
):
    async with UsersAPI(**client.api_params) as api:
        user = await api.get_user(id)
    print_users_table([user])


async def create_team_and_user(
    client: DomServerClient,
    user: CreateUser,
    category_id: Optional[int] = None,
    affiliation_id: Optional[int] = None,
    user_roles: Optional[List[int]] = None,
    enabled: bool = True,
    password_length: Optional[int] = None,
    password_pattern: Optional[str] = None,
) -> CreateUser:
    if not category_id:
        category_id = client.category_id

    if not user_roles:
        user_roles = client.user_roles

    if not user.password:
        user.password = gen_password(password_length, password_pattern)

    async with DomServerWeb(**client.api_params) as web:
        await web.login()
        if not affiliation_id and not user.affiliation:
            affiliation_id = client.affiliation_id
        elif user.affiliation:
            affiliation = await web.get_affiliation(user.affiliation)

            if affiliation:
                affiliation_id = affiliation.id
            else:
                name = user.affiliation
                affiliation = await web.create_affiliation(
                    name,
                    name,
                    client.affiliation_country,
                )
                affiliation_id = affiliation.id

        team_id, user_id = await web.create_team_and_user(
            user,
            category_id,
            affiliation_id,
            enabled,
        )
        await web.set_user_password(user_id, user.password, user_roles, enabled)

        return user


async def create_teams_and_users(
    client: DomServerClient,
    file: typer.FileText,
    category_id: Optional[int] = None,
    affiliation_id: Optional[int] = None,
    user_roles: Optional[List[int]] = None,
    enabled: bool = True,
    format: Optional[UserExportFormat] = None,
    ignore_existing: bool = False,
    delete_existing: bool = False,
    password_length: Optional[int] = None,
    password_pattern: Optional[str] = None,
) -> None:
    async with UsersAPI(**client.api_params) as api:
        users = await api.all_users()

    existing_users = [it.username for it in users]

    if not format:
        format = UserExportFormat.CSV

    input_file = file
    if format == UserExportFormat.CSV:
        input_file = file.read().replace("\ufeff", "")

    users = []
    delete_users = []
    dataset = Dataset().load(input_file, format=format.value)

    for item in dataset.dict:
        item["email"] = None if not item.get("email") else item["email"]
        user = CreateUser(**item)

        if user.username in existing_users:
            if delete_existing:
                delete_users.append(user.username)

            if ignore_existing:
                typer.echo(f"User {user.username} is ignored")
                continue

        users.append(user)

    if delete_users:
        async with DomServerWeb(**client.api_params) as web:
            await web.login()
            typer.echo("Delete existing users.")
            await web.delete_users(delete_users)
            typer.echo("Delete existing teams.")
            await web.delete_teams(delete_users)

    new_users = []
    with typer.progressbar(users) as progress:
        for user in progress:
            new_user = await create_team_and_user(
                client,
                user,
                category_id,
                affiliation_id,
                user_roles,
                enabled,
                password_length,
                password_pattern,
            )
            new_users.append(new_user)

    if new_users:
        file_name = format.export(new_users, name="import-users-teams-out")
        typer.echo(file_name)


async def delete_teams_and_users(
    client: DomServerClient,
    include: Optional[List[str]] = None,
    exclude: Optional[List[str]] = None,
) -> None:
    default_ignore_users = ["admin", "judgehost", client.username]

    async with UsersAPI(**client.api_params) as api:
        users = await api.all_users()

    existing_users = [it.username for it in users]

    if not exclude:
        exclude = default_ignore_users

    if not include:
        include = existing_users

    if include:
        include = list(
            filter(
                lambda it: it not in default_ignore_users,
                include,
            )
        )

    async with DomServerWeb(**client.api_params) as web:
        await web.login()
        typer.echo("Delete users.")
        await web.delete_users(include, exclude)
        typer.echo("Delete teams.")
        await web.delete_teams(include, exclude)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['domjudge_tool_cli',
 'domjudge_tool_cli.commands',
 'domjudge_tool_cli.commands.emails',
 'domjudge_tool_cli.commands.general',
 'domjudge_tool_cli.commands.scoreboard',
 'domjudge_tool_cli.commands.submissions',
 'domjudge_tool_cli.commands.users',
 'domjudge_tool_cli.models',
 'domjudge_tool_cli.services',
 'domjudge_tool_cli.services.v4',
 'domjudge_tool_cli.utils',
 'domjudge_tool_cli.utils.email']

package_data = \
{'': ['*'], 'domjudge_tool_cli': ['templates/csv/*', 'templates/email/*']}

install_requires = \
['aiofiles>=0.8.0,<0.9.0',
 'beautifulsoup4>=4.10.0,<5.0.0',
 'httpx>=0.19.0,<0.20.0',
 'openpyxl>=3.0.9,<4.0.0',
 'pydantic[dotenv,email]>=1.8.2,<2.0.0',
 'tablib[all]>=3.0.0,<4.0.0',
 'typer>=0.4.0,<0.5.0']

entry_points = \
{'console_scripts': ['domjudge-tool-cli = domjudge_tool_cli:app']}

setup_kwargs = {
    'name': 'domjudge-tool-cli',
    'version': '0.1.0',
    'description': 'DomJudge dom server cli tool.',
    'long_description': None,
    'author': 'Jason Xie',
    'author_email': 'x5758x@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from typing import List, Optional

from pydantic import BaseModel


class Team(BaseModel):
    group_ids: List[str]
    affiliation: Optional[str]
    nationality: Optional[str]
    id: str
    icpc_id: Optional[str]
    name: str
    display_name: Optional[str]
    organization_id: Optional[str]
    members: Optional[str]



from .general import GeneralAPI
from .problems import ProblemsAPI
from .submissions import SubmissionsAPI
from .teams import TeamsAPI
from .users import UsersAPI
from .web import DomServerWeb

__all__ = (
    "GeneralAPI",
    "UsersAPI",
    "DomServerWeb",
    "SubmissionsAPI",
    "TeamsAPI",
    "ProblemsAPI",
)



from pathlib import Path
from typing import Any, Dict, Optional

import typer

from domjudge_tool_cli.models import DomServerClient
from domjudge_tool_cli.services.v4 import DomServerWeb, GeneralAPI


async def get_version(client: DomServerClient):
    async with GeneralAPI(**client.api_params) as api:
        version = await api.version()
    message = typer.style(
        f"Success connect API v{version}.",
        fg=typer.colors.GREEN,
        bold=True,
    )
    typer.echo(message)


async def check_login_website(client: DomServerClient):
    async with DomServerWeb(**client.api_params) as web:
        await web.login()
        message = typer.style(
            f"Success connect DomJudge website.",
            fg=typer.colors.GREEN,
            bold=True,
        )
        typer.echo(message)


def create_config(
    host: str,
    username: str,
    password: str,
    disable_ssl: bool = typer.Option(False),
    timeout: Optional[float] = None,
    max_connections: Optional[int] = None,
    max_keepalive_connections: Optional[int] = None,
) -> DomServerClient:
    typer.echo("*" * len(password))
    dom_server = DomServerClient(
        host=host,
        username=username,
        password=password,
        disable_ssl=disable_ssl or False,
        timeout=timeout,
        max_connections=max_connections,
        max_keepalive_connections=max_keepalive_connections,
    )
    with open("domserver.json", "wb+") as f:
        f.write(dom_server.json().encode())
        typer.echo("Success config Dom Server.")

    return dom_server


def read_config(path: Optional[Path] = None) -> DomServerClient:
    if not path:
        path = Path("domserver.json")

    if path.exists() and path.is_file():
        client = DomServerClient.parse_file(path)
        return client

    raise FileNotFoundError(path)


def update_config(
    dom_server: DomServerClient,
    **kwargs: Dict[str, Any],
) -> DomServerClient:
    for k, v in kwargs.items():
        if hasattr(dom_server, k):
            setattr(dom_server, k, v)

    with open("domserver.json", "wb+") as f:
        f.write(dom_server.json().encode())
        typer.echo("Success update config.")

    return dom_server



from domjudge_tool_cli.services.api_client import APIClient


class V4Client(APIClient):
    API_VERSION = "v4"
    API_PREFIX = f"/api/{API_VERSION}"

    def make_resource(self, path: str) -> str:
        return f"{self.API_PREFIX}{path}"
